using System;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace MEdiaSpaceCanmore
{
    internal partial class CustomForm : Form
    {
        #region Code that provides hooks for the WndProc internals

        private const int WM_SYSCOMMAND = 0x0112;
        private const int SC_MINIMIZE = 0xF020;
        private const int SC_MAXIMIZE = 0xF030;

        [Serializable]
        [ComVisible(true)]
        public delegate bool FormMaximizedEventHandler(object sender, EventArgs e);

        [Serializable]
        [ComVisible(true)]
        public delegate bool FormMinimizedEventHandler(object sender, EventArgs e);

        public event FormMinimizedEventHandler FormMinimized;
        public event FormMaximizedEventHandler FormMaximized;

        private bool OnMinimized(EventArgs e)
        {
            if (FormMinimized != null)
                return FormMinimized(this, e);
            else
                return true;
        }

        private bool OnMaximized(EventArgs e)
        {
            if (FormMaximized != null)
                return FormMaximized(this, e);
            else
                return true;
        }

        protected override void WndProc(ref Message m)
        {
            switch (m.Msg)
            {
                case WM_SYSCOMMAND:
                    switch (m.WParam.ToInt32())
                    {
                        case SC_MINIMIZE:
                            if (!OnMinimized(EventArgs.Empty))
                                return;
                            else
                                break;

                        case SC_MAXIMIZE:
                            if (!OnMaximized(EventArgs.Empty))
                                return;
                            else
                                break;

                        default:
                            break;
                    }
                    break;

                default:
                    break;
            }

            base.WndProc(ref m);
        }

        #endregion


        //
        // Thread-safe UI-updating delegates
        //
        internal delegate void SetPictureBoxImage(PictureBox pbox, Image image);
        internal delegate void SetTextBoxText(TextBox tbox, String text);
        internal delegate void SetControlEnbabled(Control control, bool enabled);


        //
        // Thread-safe UI updaters
        //
        internal void DisplayImageInPictureBox(PictureBox pbox, Image image)
        {
            if (pbox.InvokeRequired)
            {
                SetPictureBoxImage theDelegate = new SetPictureBoxImage(DisplayImageInPictureBox);
                BeginInvoke(theDelegate, new object[] { pbox, image });
            }
            else
                pbox.Image = image;
        }

        internal void DisplayTextInTextBox(TextBox tbox, String text)
        {
            if (tbox.InvokeRequired)
            {
                SetTextBoxText theDelegate = new SetTextBoxText(DisplayTextInTextBox);
                BeginInvoke(theDelegate, new object[] { tbox, text });
            }
            else
                tbox.Text = text;
        }

        internal void EnableControl(Control control, bool enabled)
        {
            if (control.InvokeRequired)
            {
                SetControlEnbabled theDelegate = new SetControlEnbabled(EnableControl);
                BeginInvoke(theDelegate, new object[] { control, enabled });
            }
            else
                control.Enabled = enabled;
        }
    }
}
