using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace MEdiaSpaceCanmore
{
    internal sealed partial class LargeForm : CustomForm
    {
        // UI status variables
        private const int PBOX_MARGIN = 5;       // The outgoingPictureBox's drag margin
        private bool _isDragging;
        private int _dragX, _dragY;


        public LargeForm()
        {
            InitializeComponent();

            _isDragging = false;
        }


        // 
        // UI component handlers
        //
        private void LargeForm_Load(object sender, EventArgs e)
        {
            this.FormMinimized += new FormMinimizedEventHandler(LargeForm_FormMinimized);
            this.FormClosed += new FormClosedEventHandler(LargeForm_FormClosed);
        }

        private bool LargeForm_FormMinimized(object sender, EventArgs e)
        {
            FormController.SwitchToMedium();
            return false;
        }

        private void LargeForm_FormClosed(object sender, EventArgs e)
        {
            FormController.Shutdown();
        }

        private void connectButton_Click(object sender, EventArgs e)
        {
            FormController.OnConnectClicked();
        }

        private void doorButton_Click(object sender, EventArgs e)
        {
            FormController.OnDoorClicked();
        }

        void outgoingPictureBox_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            _isDragging = false;
        }

        void outgoingPictureBox_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            if (_isDragging)
            {
                int newTop = outgoingPictureBox.Top + (e.Y - _dragY);
                int newLeft = outgoingPictureBox.Left + (e.X - _dragX);

                newTop = Math.Max(newTop, incomingPictureBox.Top + PBOX_MARGIN);
                newTop = Math.Min(newTop, incomingPictureBox.Bottom - PBOX_MARGIN - outgoingPictureBox.Height);

                newLeft = Math.Max(newLeft, incomingPictureBox.Left + PBOX_MARGIN);
                newLeft = Math.Min(newLeft, incomingPictureBox.Right - PBOX_MARGIN - outgoingPictureBox.Width);

                outgoingPictureBox.Top = newTop;
                outgoingPictureBox.Left = newLeft;
            }
        }

        void outgoingPictureBox_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            _isDragging = true;

            _dragX = e.X;
            _dragY = e.Y;
        }

        private void chatInputBox_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                FormController.OnMessageEntered(chatInputBox.Text);
                chatInputBox.Text = "";
            }
        }

        private void documentButton_Click(object sender, EventArgs e)
        {
            FormController.OnDocumentClicked();
        }

        private void phoneButton_Click(object sender, EventArgs e)
        {
            FormController.OnPhoneClicked();
        }

        private void toolsButton_Click(object sender, EventArgs e)
        {
            FormController.OnToolsClicked();
        }


        //
        // Content display functions
        //
        public void SetConnected(bool connected)
        {
            if (connected)
                connectButton.Image = global::MEdiaSpaceCanmore.Properties.Resources.disconnect;
            else
                connectButton.Image = global::MEdiaSpaceCanmore.Properties.Resources.connect;
        }

        public void setDoorEnabled(bool enabled)
        {
            // Have to use delegate magic since this can be a cross-thread call
            EnableControl(doorButton, enabled);
        }

        public void SetUIEnabled(bool enabled)
        {
            // Have to use delegate magic since this can be a cross-thread call
            EnableControl(documentButton, enabled);
            EnableControl(toolsButton, enabled);
        }

        public void SetCallDisabled(bool disabled)
        {
            // Have to use delegate magic since this can be a cross-thread call
            EnableControl(phoneButton, !disabled);
        }

        public void SetCallStatus(bool talking)
        {
            if (talking)
                phoneButton.Image = global::MEdiaSpaceCanmore.Properties.Resources.redphone;
            else
                phoneButton.Image = global::MEdiaSpaceCanmore.Properties.Resources.greenphone;
        }

        public void SetMotionDetection(int value, DateTime timestamp)
        {
        }

        public void SetRemoteImage(Image image)
        {
            DisplayImageInPictureBox(incomingPictureBox, image);
        }

        public void SetLocalImage(Image image)
        {
            if (null != image)
                image.RotateFlip(RotateFlipType.Rotate180FlipY);

            DisplayImageInPictureBox(outgoingPictureBox, image);
        }

        public void ClearChatSpool()
        {
            DisplayTextInTextBox(chatOutputBox, "");
        }

        public void AddChatMessage(String sender, String message)
        {
            DisplayTextInTextBox(chatOutputBox, chatOutputBox.Text + sender + ": " + message + System.Environment.NewLine);
        }
   }
}